/*
 * metadrums.c
 *
 * Copyright (C) 2003 Conrad Parker <conrad@metadecks.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 *
 */  

#include "config.h"
#include "metadrums.h"

#include <mash.h>

#define IM PACKAGE_DATA_DIR "/"

#define TINY_CURSOR

#define WIDTH 640
#define HEIGHT 480 

#define SAMPLES_DIR PACKAGE_DATA_DIR "/"

#define KICK_PATH SAMPLES_DIR "housekick1.wav"
#define SNARE_PATH SAMPLES_DIR "crazysnare1.wav"
#define CONGAHIGH_PATH SAMPLES_DIR "congahigh1.wav"
#define CONGALOW_PATH SAMPLES_DIR "congalow1.wav"
#define CLOSEDHAT_PATH SAMPLES_DIR "closedhat10.wav"
#define OPENHAT_PATH SAMPLES_DIR "hr16openhat.wav"
#define CLAP_PATH SAMPLES_DIR "clap45.wav"
#define CLAVE_PATH SAMPLES_DIR "hr16clave.wav"

#define LOOP_LEN 192
#define QUANT 3

#define DEFAULT_BPM 140

#define FONT "notepad"
#define FONT_SIZE 12

typedef void (*Evas_Event_Callback) (void *data, Evas *e, Evas_Object *obj, void *event_info);

static char * progname = "MetaDrums";

Evas *evas = NULL;
Evas_Object * cursor;

int win_w = WIDTH;
int win_h = HEIGHT;

MASH * mash;
MashDeck * drums_deck, * mix_deck;

MashSquareTone * square1, * square2;
MashBase * kick_sf, * snare_sf, * congahigh_sf, * congalow_sf,
  * closedhat_sf, * openhat_sf, * clap_sf, * clave_sf;

MashTrack * mix_track;
MashLayer * drums_layer, * delay_layer, * vcf303_layer, * monitor_layer;

MashBase * delay1_fx, * delay2_fx, * vcf303_fx;
MashMonitor * monitor;

float delay1_x, delay2_x;

MashBase *
sndfile_reader_new (char * pathname)
{
  static MashPlugin * sf_plugin = NULL;
  MashBase * base;
  CDSet * sf_parms;
  int sf_path_key;

  if (sf_plugin == NULL) {
    sf_plugin = mash_find_plugin (mash, "builtin::sndfile_reader");
  }

  if (sf_plugin == NULL) {
    fprintf (stderr, "%s: Could not find mash sndfile reader plugin\n",
	     progname);
    exit (1);
  }

  sf_parms = cd_set_new (mash);
  sf_path_key = mash_get_init_parameter_key (mash, sf_plugin, "path");
  sf_parms = cd_set_insert (mash, sf_parms, sf_path_key,
			    CD_STRING(pathname));

  base = mash_new (mash, sf_plugin, sf_parms);

  return base;
}

static MashPlugin * delay_plugin = NULL;
static MashPlugin * vcf303_plugin = NULL;

static void
setup_decks (void)
{
  MashEnvelope * env;
  MashSound * sound;

  mash = mash_init ();
  mash_set_tempo (mash, DEFAULT_BPM);
  mash_set_channels (mash, MASH_MONO);

  drums_deck = mash_deck_new (mash);

  square1 = mash_squaretone_new (mash, 220);
  square2 = mash_squaretone_new (mash, 238);

  kick_sf = sndfile_reader_new (KICK_PATH);
  snare_sf = sndfile_reader_new (SNARE_PATH);
  congahigh_sf = sndfile_reader_new (CONGAHIGH_PATH);
  congalow_sf = sndfile_reader_new (CONGALOW_PATH);
  closedhat_sf = sndfile_reader_new (CLOSEDHAT_PATH);
  openhat_sf = sndfile_reader_new (OPENHAT_PATH);
  clap_sf = sndfile_reader_new (CLAP_PATH);
  clave_sf = sndfile_reader_new (CLAVE_PATH);

  mix_deck = mash_deck_new (mash);
  mix_track = mash_add_track (mash, mix_deck);
  
  drums_layer = mash_add_layer_ontop (mash, mix_track, MASH_TIME_BEAT24S);
  mash_add_sound (mash, (MashBase *)drums_deck, drums_layer,
		  MASH_BEAT24S(0), MASH_BEAT24S(LOOP_LEN));

  env = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_add_point (mash, env, MASH_BEAT24S(0), 0.2);

  delay_plugin = mash_find_plugin (mash, "ladspa::1061");

  if (delay_plugin == NULL) {
    printf ("%s: Could not find Feedback Delay 5s LADSPA plugin (id 1061).\n"
	    "You should install CMT from http://www.ladspa.org/cmt\n",
	    progname);
  } else {

    delay1_fx = mash_new (mash, delay_plugin, CD_EMPTY_SET);
    mash_set_parameter (mash, delay1_fx, 0, CD_FLOAT(0.1)); /* Delay (s) */
    mash_set_parameter (mash, delay1_fx, 1, CD_FLOAT(0.2)); /* Dry/Wet */
    mash_set_parameter (mash, delay1_fx, 2, CD_FLOAT(0.2)); /* Feedback */

    delay_layer = mash_add_layer_ontop (mash, mix_track, MASH_TIME_BEAT24S);
    sound = mash_add_sound (mash, delay1_fx, delay_layer,
			    MASH_BEAT24S(0), MASH_BEAT24S(LOOP_LEN));
    /*  mash_sound_set_blend_envelope (mash, sound, env);*/
  
    delay2_fx = mash_new (mash, delay_plugin, CD_EMPTY_SET);
    mash_set_parameter (mash, delay2_fx, 0, CD_FLOAT(0.2)); /* Delay (s) */
    mash_set_parameter (mash, delay2_fx, 1, CD_FLOAT(0.1)); /* Dry/Wet */
    mash_set_parameter (mash, delay2_fx, 2, CD_FLOAT(0.1)); /* Feedback */

    delay_layer = mash_add_layer_ontop (mash, mix_track, MASH_TIME_BEAT24S);
    sound = mash_add_sound (mash, delay2_fx, delay_layer,
			    MASH_BEAT24S(0), MASH_BEAT24S(LOOP_LEN));
    /*mash_sound_set_blend_envelope (mash, sound, env);*/
  }
    
  vcf303_plugin = mash_find_plugin (mash, "ladspa::1224");

  if (vcf303_plugin == NULL) {
    printf ("%s: Could not find VCF 303 LADSPA plugin (id 1224).\n"
	    "You should install CMT from http://www.ladspa.org/cmt\n",
	    progname);
  } else {
    vcf303_fx = mash_new (mash, vcf303_plugin, CD_EMPTY_SET);
    mash_set_parameter (mash, vcf303_fx, 0, CD_FLOAT(1)); /* Trigger */
    mash_set_parameter (mash, vcf303_fx, 1, CD_FLOAT(0.6)); /* Cutoff */
    mash_set_parameter (mash, vcf303_fx, 2, CD_FLOAT(0.7)); /* Resonance */
    mash_set_parameter (mash, vcf303_fx, 3, CD_FLOAT(0.4)); /* Env. Mod. */
    mash_set_parameter (mash, vcf303_fx, 4, CD_FLOAT(0.3)); /* Decay */
  
    vcf303_layer = mash_add_layer_ontop (mash, mix_track, MASH_TIME_BEAT24S);
    mash_add_sound (mash, vcf303_fx, vcf303_layer,
		    MASH_BEAT24S(0), MASH_BEAT24S(LOOP_LEN));
  }

  monitor = mash_monitor_new (mash);
  monitor_layer = mash_add_layer_ontop (mash, mix_track, MASH_TIME_BEAT24S);
  mash_add_sound (mash, (MashBase *)monitor, monitor_layer,
		  MASH_BEAT24S(0), MASH_BEAT24S(LOOP_LEN));
}

#if 0
void
zoom_in (void)
{
  double x, y, w, h;
  int sx, sy, sw, sh;

  evas_output_viewport_get (evas, &x, &y, &w, &h);

  x += w/4;
  w /= 2;

  evas_output_viewport_set (evas, x, y, w, h);

  sx = evas_coord_world_x_to_screen (evas, x);
  sy = evas_coord_world_y_to_screen (evas, y);
  sw = evas_coord_world_x_to_screen (evas, w);
  sh = evas_coord_world_y_to_screen (evas, h);

  evas_damage_rectangle_add (evas, sx, sy, sw, sh);
}

void
zoom_out (void)
{
  double x, y, w, h;
  int sx, sy, sw, sh;

  evas_output_viewport_get (evas, &x, &y, &w, &h);
  x -= w/2;
  w *= 2;

  if (w >= WIDTH) {
    x = 0;
    w = WIDTH;
  }

  evas_output_viewport_set (evas, x, y, w, h);

  sx = evas_coord_world_x_to_screen (evas, x);
  sy = evas_coord_world_y_to_screen (evas, y);
  sw = evas_coord_world_x_to_screen (evas, w);
  sh = evas_coord_world_y_to_screen (evas, h);

  evas_damage_rectangle_add (evas, sx, sy, sw, sh);
}
#endif

static void
disable_rect (double x, double y, double w, double h)
{
  Evas_Object * ob;

  ob = evas_object_rectangle_add (evas);
  evas_object_move (ob, x, y);
  evas_object_resize (ob, w, h);
  evas_object_color_set (ob, 200, 200, 200, 160);
  evas_object_layer_set (ob, 4);
  evas_object_show (ob);
}

void
delay_reset (MashBase * delay_fx, float canvas_x)
{
 int beat;
 MashTime ts;
 float value;
 
 beat = LOOP_LEN * canvas_x / WIDTH;
 ts = mash_time_convert (mash, MASH_BEAT24S(beat), MASH_TIME_BEAT24S,
			 MASH_TIME_SECONDS);
 mash_set_parameter (mash, delay_fx, 0, CD_FLOAT(ts.seconds)); 
}

static Evas_Object * tempo_label;

void
tempo_set_x (Evas_Object * slider, float canvas_x)
{
  MashTempo new_tempo;
  static char buf[32];

  if (canvas_x > 0 && canvas_x < WIDTH/2) {
    new_tempo =  60 + 180 * (canvas_x / (WIDTH/2));
    snprintf (buf, 32, "Tempo: %.1f BPM", new_tempo);
    
    mash_set_tempo (mash, new_tempo);
    
    evas_object_text_text_set (tempo_label, buf);
    evas_object_resize (slider, canvas_x, HEIGHT/16 - 16);

    delay_reset (delay1_fx, delay1_x);
    delay_reset (delay2_fx, delay2_x);
  }
}

void
tempo_mouse_down(void *data, Evas * e, Evas_Object * obj,
		  Evas_Event_Mouse_Down * ev)
{
  Evas_Object * slider = (Evas_Object *)data;

  evas_object_data_set (obj, "moving", (void *)1);
  tempo_set_x (slider, ev->canvas.x);
}

void
tempo_mouse_move (void *data, Evas * e, Evas_Object * obj,
		  Evas_Event_Mouse_Move * ev)
{
  Evas_Object * slider = (Evas_Object *)data;

  if (evas_object_data_get (obj, "moving") != NULL) {
    tempo_set_x (slider, ev->cur.canvas.x);
  }
}

void
tempo_mouse_up (void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Move * ev)
{
  evas_object_data_set (obj, "moving", NULL);
}

static void
setup_tempo_slider (void)
{
  double track_y;
  Evas_Object * bg, * ob;
  CDScalar value;
  char buf[32];
  MashTempo tempo = mash_get_tempo (mash);

  track_y = 3 * HEIGHT/16;

  ob = evas_object_image_add (evas);
  evas_object_image_file_set(ob, IM "track_bg.png", NULL);
  evas_object_move(ob, 0, track_y);
  evas_object_resize(ob, WIDTH/2, HEIGHT/16);
  evas_object_image_fill_set(ob, 0, 0, WIDTH/2, HEIGHT/16);
  evas_object_color_set (ob, 240, 240, 240, 255);
  evas_object_layer_set(ob, 0);
  evas_object_show(ob);
  
  bg = ob;

  ob = evas_object_rectangle_add (evas);
  evas_object_move (ob, 0, track_y + 8);
  evas_object_resize (ob, ((tempo - 60) / 180) * WIDTH/2, HEIGHT/16 - 16);
  evas_object_color_set (ob, 250, 250, 120, 100);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

  evas_object_event_callback_add(bg, EVAS_CALLBACK_MOUSE_DOWN,
				 (Evas_Event_Callback)tempo_mouse_down, ob);
  evas_object_event_callback_add(bg, EVAS_CALLBACK_MOUSE_UP,
				 (Evas_Event_Callback)tempo_mouse_up, ob);
  evas_object_event_callback_add(bg, EVAS_CALLBACK_MOUSE_MOVE,
				 (Evas_Event_Callback)tempo_mouse_move, ob);

  snprintf (buf, 32, "Tempo: %.1f BPM", tempo);

  ob = evas_object_text_add (evas);
  evas_object_text_font_set (ob, FONT, FONT_SIZE);
  evas_object_text_text_set (ob, buf);
  evas_object_move (ob, 10, track_y+8);
  evas_object_layer_set (ob, 1);
  evas_object_color_set (ob, 0, 0, 0, 255);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

  tempo_label = ob;
}

/* VCF 303 */

typedef struct _MD_VCF303 MD_VCF303;

struct _MD_VCF303 {
  MashBase * vcf303_fx;
  int param_key;
  Evas_Object * slider;
};

void
vcf303_set_x (MD_VCF303 * md, float canvas_x)
{
  int beat;
  MashTime ts;
  float value;

  mash_set_parameter (mash, md->vcf303_fx, md->param_key,
		      CD_FLOAT((canvas_x - WIDTH/2) / (WIDTH/2)));
  evas_object_resize (md->slider, canvas_x - WIDTH/2, HEIGHT/16 - 16);
}

void
vcf303_mouse_down(void *data, Evas * e, Evas_Object * obj,
		  Evas_Event_Mouse_Down * ev)
{
  MD_VCF303 * md = (MD_VCF303 *)data;

  evas_object_data_set (obj, "moving", (void *)1);

  vcf303_set_x (md, ev->canvas.x);
}

void
vcf303_mouse_move (void *data, Evas * e, Evas_Object * obj,
		  Evas_Event_Mouse_Move * ev)
{
  MD_VCF303 * md = (MD_VCF303 *)data;

  if (evas_object_data_get (obj, "moving") != NULL) {
    vcf303_set_x (md, ev->cur.canvas.x);
  }
}

void
vcf303_mouse_up (void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Move * ev)
{
  evas_object_data_set (obj, "moving", NULL);
}

static void
setup_vcf303_slider (int index, char * name)
{
  MD_VCF303 * md;
  double track_y;
  Evas_Object * ob;
  CDScalar value;

  md = malloc (sizeof(MD_VCF303));
  md->vcf303_fx = vcf303_fx;
  md->param_key = index;

  track_y = (index-1) * HEIGHT/16;

  ob = evas_object_image_add (evas);
  evas_object_image_file_set(ob, IM "track_bg.png", NULL);
  evas_object_move(ob, WIDTH/2, track_y);
  evas_object_resize(ob, WIDTH/2, HEIGHT/16);
  evas_object_image_fill_set(ob, 0, 0, WIDTH/2, HEIGHT/16);
  evas_object_color_set (ob, 250, 240, 220, 255);
  evas_object_layer_set(ob, 0);
  evas_object_show(ob);
  evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_DOWN,
				 (Evas_Event_Callback)vcf303_mouse_down, md);
  evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_UP,
				 (Evas_Event_Callback)vcf303_mouse_up, md);
  evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_MOVE,
				 (Evas_Event_Callback)vcf303_mouse_move, md);

  value = mash_get_parameter (mash, vcf303_fx, index);

  if (vcf303_plugin != NULL) {
    ob = evas_object_rectangle_add (evas);
    evas_object_move (ob, WIDTH/2, track_y + 8);
    evas_object_resize (ob, value.s_float * WIDTH/2, HEIGHT/16 - 16);
    evas_object_color_set (ob, 250, 250, 120, 100);
    evas_object_pass_events_set (ob, 1);
    evas_object_show (ob);
    md->slider = ob;
  }

  ob = evas_object_text_add (evas);
  evas_object_text_font_set (ob, FONT, FONT_SIZE);
  evas_object_text_text_set (ob, name);
  evas_object_move (ob, WIDTH/2 + 10, track_y+8);
  evas_object_layer_set (ob, 1);
  evas_object_color_set (ob, 0, 0, 0, 255);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

}

static void
setup_vcf303 (void)
{
  Evas_Object * ob;

  /* These names can be read from the plugin but this code is DRUNK */
  setup_vcf303_slider (1, "Cutoff");
  setup_vcf303_slider (2, "Resonance");
  setup_vcf303_slider (3, "Envelope Modulation");
  setup_vcf303_slider (4, "Decay");

  ob = evas_object_text_add (evas);
  evas_object_text_font_set (ob, FONT, FONT_SIZE*4.4);
  evas_object_text_text_set (ob, "VCF303");
  evas_object_move (ob, WIDTH/2 + 30, 30);
  evas_object_layer_set (ob, 1);
  evas_object_color_set (ob, 200, 200, 200, 100);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

  if (vcf303_plugin == NULL) {
    disable_rect (WIDTH/2, 0, WIDTH/2, HEIGHT/4);

    ob = evas_object_text_add (evas);
    evas_object_text_font_set (ob, FONT, FONT_SIZE*2);
    evas_object_text_text_set (ob, "Plugin not found");
    evas_object_move (ob, WIDTH/2 + 40, HEIGHT/8 - 20);
    evas_object_layer_set (ob, 5);
    evas_object_color_set (ob, 200, 20, 20, 200);
    evas_object_pass_events_set (ob, 1);
    evas_object_show (ob);

  }
}

/* Scrub */

void
scrub_set_x (float canvas_x)
{
  MashCount length, offset, n = 0;

  length = mash_length (mash, mix_deck);

  mash_seek (mash, mix_deck, (MashCount)(length * canvas_x / WIDTH), SEEK_SET);
}

void
scrub_mouse_down(void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Down * ev)
{
  evas_object_data_set (obj, "moving", (void *)1);

  scrub_set_x (ev->canvas.x);
}

void
scrub_mouse_move (void *data, Evas * e, Evas_Object * obj,
		  Evas_Event_Mouse_Move * ev)
{
  if (evas_object_data_get (obj, "moving") != NULL) {
    scrub_set_x (ev->cur.canvas.x);
  }
}

void
scrub_mouse_up (void *data, Evas * e, Evas_Object * obj,
		Evas_Event_Mouse_Move * ev)
{
  evas_object_data_set (obj, "moving", NULL);
}

static void
setup_scrub (void)
{
  double track_y;
  Evas_Object * ob;

  track_y = HEIGHT/4;

  ob = evas_object_image_add (evas);
  evas_object_image_file_set(ob, IM "track_bg.png", NULL);
  evas_object_move(ob, 0, track_y);
  evas_object_resize(ob, WIDTH, (HEIGHT/2)/4);
  evas_object_image_fill_set(ob, 0, 0, WIDTH/4, (HEIGHT/2)/8);
  evas_object_color_set (ob, 220, 250, 220, 255);
  evas_object_layer_set(ob, 0);
  evas_object_show(ob);
  evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_DOWN,
				 (Evas_Event_Callback)scrub_mouse_down, NULL);
  evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_UP,
				 (Evas_Event_Callback)scrub_mouse_up, NULL);
  evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_MOVE,
				 (Evas_Event_Callback)scrub_mouse_move, NULL);

  ob = evas_object_text_add (evas);
  evas_object_text_font_set (ob, FONT, FONT_SIZE*1.4);
  evas_object_text_text_set (ob, "Scrub");
  evas_object_move (ob, 10, track_y+12);
  evas_object_layer_set (ob, 1);
  evas_object_color_set (ob, 100, 100, 100, 200);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

}

/* Delay */

typedef struct _MD_Delay MD_Delay;

struct _MD_Delay {
  MashBase * delay_fx;
  Evas_Object * slider;
  float * delay_x;
};

void
delay_set_x (MD_Delay * md, float canvas_x)
{
  *md->delay_x = canvas_x;
  delay_reset (md->delay_fx, canvas_x);
  evas_object_resize (md->slider, canvas_x, HEIGHT/16 - 16);  
}

void
delay_mouse_down(void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Down * ev)
{
  MD_Delay * md = (MD_Delay *)data;

#if 0
  if (ev->button == 4) {
    zoom_in ();
  } else if (ev->button == 5) {
    zoom_out ();
  } else {
    evas_object_data_set (obj, "moving", (void *)1);
    
    delay_set_x (md, ev->canvas.x);
  }
#else
  evas_object_data_set (obj, "moving", (void *)1);
  delay_set_x (md, ev->canvas.x);
#endif
}

void
delay_mouse_move (void *data, Evas * e, Evas_Object * obj,
		  Evas_Event_Mouse_Move * ev)
{
  MD_Delay * md = (MD_Delay *)data;

  if (evas_object_data_get (obj, "moving") != NULL) {
    delay_set_x (md, ev->cur.canvas.x);
  }
}

void
delay_mouse_up (void *data, Evas * e, Evas_Object * obj,
		Evas_Event_Mouse_Move * ev)
{
  evas_object_data_set (obj, "moving", NULL);
}


static void
setup_delay (int index, MashBase * delay_fx, float * delay_x)
{
  MD_Delay * md;
  MashTrack * track;
  MashLayer * layer;
  Evas_Object * ob;
  int iw, ih;
  double track_y;
  CDScalar delay_seconds;
  MashTime loop_seconds;

  track_y = 3*HEIGHT/8 + index * HEIGHT / (2 * 8);

  md = malloc (sizeof(MD_Delay));

  md->delay_fx = delay_fx;
  md->delay_x = delay_x;
  
  ob = evas_object_image_add (evas);
  evas_object_image_file_set(ob, IM "track_bg.png", NULL);
  evas_object_move(ob, 0, track_y);
  evas_object_resize(ob, WIDTH, (HEIGHT/2)/8);
  evas_object_image_fill_set(ob, 0, 0, WIDTH/8, (HEIGHT/2)/8);
  evas_object_color_set (ob, 250, 240, 220, 255);
  evas_object_layer_set(ob, 0);
  evas_object_show(ob);

  if (delay_plugin != NULL) {
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_DOWN,
				   (Evas_Event_Callback)delay_mouse_down, md);
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_UP,
				   (Evas_Event_Callback)delay_mouse_up, md);
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_MOVE,
				   (Evas_Event_Callback)delay_mouse_move, md);
    
    
    ob = evas_object_rectangle_add (evas);
    evas_object_move (ob, 0, track_y + 8);
    evas_object_color_set (ob, 250, 250, 120, 100);
    evas_object_pass_events_set (ob, 1);
    evas_object_show (ob);
    md->slider = ob;

    delay_seconds = mash_get_parameter (mash, delay_fx, 0);
    loop_seconds = mash_time_convert (mash, MASH_BEAT24S(LOOP_LEN),
				      MASH_TIME_BEAT24S, MASH_TIME_SECONDS);
    *delay_x = delay_seconds.s_float / loop_seconds.seconds * WIDTH;
    evas_object_resize (ob, *delay_x, HEIGHT/16 - 16);
  }

  ob = evas_object_text_add (evas);
  evas_object_text_font_set (ob, FONT, FONT_SIZE);
  evas_object_text_text_set (ob, "Delay");
  evas_object_move (ob, 10, track_y+8);
  evas_object_layer_set (ob, 1);
  evas_object_color_set (ob, 0, 0, 0, 255);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

  if (delay_plugin == NULL) {
    disable_rect (0, track_y, WIDTH, HEIGHT/16);

    ob = evas_object_text_add (evas);
    evas_object_text_font_set (ob, FONT, FONT_SIZE);
    evas_object_text_text_set (ob, "Plugin not found");
    evas_object_move (ob, 180, track_y+8);
    evas_object_layer_set (ob, 5);
    evas_object_color_set (ob, 200, 20, 20, 200);
    evas_object_pass_events_set (ob, 1);
    evas_object_show (ob);

  }

}

/* Track GUI */

typedef struct _MDTrack MDTrack;

struct _MDTrack {
  MashBase * sf_base;
  MashLayer * layer;
  double track_y;
  char * img_path;
};

void
sound_mouse_down(void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Down * ev)
{
  MashBase * sound;

  if (ev->button == 1) {
    evas_object_data_set (obj, "moving", (void *)1);
  } else if (ev->button == 3) {
    sound = evas_object_data_get (obj, "sound");
    mash_destroy (mash, sound);
    evas_object_del (obj);
  }
}

void
sound_mouse_up(void *data, Evas * e, Evas_Object * obj,
	       Evas_Event_Mouse_Up * ev)
{
  double ox, oy;
  int beat;

  evas_object_geometry_get (obj, &ox, &oy, NULL, NULL);
  beat = (int) ((LOOP_LEN/QUANT) * ox / WIDTH) * QUANT;
  evas_object_move (obj, WIDTH * beat / LOOP_LEN, oy);

  evas_object_data_set (obj, "moving", NULL);
}

void
sound_mouse_move(void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Move * ev)
{
  MashBase * sound;
  double ox, oy, nx;
  int beat;
  
  if (evas_object_data_get (obj, "moving") != NULL) {
    evas_object_geometry_get (obj, &ox, &oy, NULL, NULL);
    sound = evas_object_data_get (obj, "sound");

    beat = (int) ((LOOP_LEN/QUANT) * ev->cur.canvas.x / WIDTH) * QUANT;
    mash_sound_move (mash, sound, MASH_BEAT24S(beat));

    evas_object_move (obj, WIDTH * beat / LOOP_LEN, oy);
  }
}

void
track_mouse_down(void *data, Evas * e, Evas_Object * obj,
		 Evas_Event_Mouse_Down * ev)
{
  MDTrack * md = (MDTrack *)data;
  Evas_Object * ob;
  int iw, ih;
  MashBase * sound;
  int beat;

  if (ev->button == 1) {
    ob = evas_object_image_add(evas);
    evas_object_image_file_set(ob, md->img_path, NULL);
    evas_object_image_size_get(ob, &iw, &ih);
    evas_object_move(ob, ev->canvas.x - iw/2, md->track_y + 2);
    evas_object_resize(ob, (int)(iw/2), HEIGHT/16-4);
    evas_object_image_fill_set(ob, 0, 0, (int)(iw/2), HEIGHT/16-4);
    evas_object_color_set (ob, 255, 255, 255, 140);
    evas_object_layer_set(ob, 1);
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_DOWN,
				   (Evas_Event_Callback)sound_mouse_down,
				   NULL);
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_UP,
				   (Evas_Event_Callback)sound_mouse_up,
				   NULL);
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_MOVE,
				   (Evas_Event_Callback)sound_mouse_move,
				   NULL);

    beat = (int) ((LOOP_LEN/QUANT) * ev->canvas.x / WIDTH) * QUANT;

    sound = mash_add_sound (mash, md->sf_base, md->layer,
			    MASH_BEAT24S(beat), MASH_BEAT24S(12));

    evas_object_move (ob, WIDTH * beat / LOOP_LEN, md->track_y + 2);
    evas_object_show(ob);

    evas_object_data_set (ob, "sound", sound);
  }
}

static void
setup_track (int index, char * name, MashBase * sf_base, MashPCM gain,
	     char * img_path)
{
  MDTrack * md;
  MashTrack * track;
  MashLayer * layer;
  Evas_Object * ob;
  int iw, ih;
  double track_y;

  track = mash_add_track (mash, drums_deck);
  mash_track_set_gain (mash, track, gain);

  layer = mash_add_layer_ontop (mash, track, MASH_TIME_BEAT24S);

  track_y = HEIGHT/2 + index * HEIGHT / (2 * 8);

  if (sf_base != NULL) {
    md = malloc (sizeof(MDTrack));
    md->sf_base = sf_base;
    md->layer = layer;
    md->track_y = track_y;
    md->img_path = img_path;
  }

  ob = evas_object_image_add (evas);
  evas_object_image_file_set(ob, IM "track_bg.png", NULL);
  evas_object_move(ob, 0, track_y);
  evas_object_resize(ob, WIDTH, (HEIGHT/2)/8);
  evas_object_image_fill_set(ob, 0, 0, WIDTH/8, (HEIGHT/2)/8);
  evas_object_layer_set(ob, 0);
  if (index % 2) {
    evas_object_color_set (ob, 240, 240, 240, 255);
  } else {
    evas_object_color_set (ob, 220, 220, 220, 255);
  }
  evas_object_show(ob);

  if (sf_base != NULL) {
    evas_object_event_callback_add(ob, EVAS_CALLBACK_MOUSE_DOWN,
				   (Evas_Event_Callback)track_mouse_down, md);
  }

  ob = evas_object_text_add (evas);
  evas_object_text_font_set (ob, FONT, FONT_SIZE);
  evas_object_text_text_set (ob, name);
  evas_object_move (ob, 10, track_y+8);
  evas_object_layer_set (ob, 1);
  evas_object_color_set (ob, 60, 60, 60, 200);
  evas_object_pass_events_set (ob, 1);
  evas_object_show (ob);

  if (sf_base == NULL) {
    disable_rect (0, track_y, WIDTH, HEIGHT/16);

    ob = evas_object_text_add (evas);
    evas_object_text_font_set (ob, FONT, FONT_SIZE);
    evas_object_text_text_set (ob, "Sample not found");
    evas_object_move (ob, 180, track_y+8);
    evas_object_layer_set (ob, 5);
    evas_object_color_set (ob, 200, 20, 20, 200);
    evas_object_pass_events_set (ob, 1);
    evas_object_show (ob);

  }
}

void
setup_gui (void)
{
   Evas_Object        *ob;
   int                 iw, ih;

   evas_font_path_prepend(evas, PACKAGE_DATA_DIR);
#if 1
   evas_image_cache_set(evas, 1024 * 1024);
   evas_font_cache_set(evas, 256 * 1024);
#else
   evas_image_cache_set(evas, 0);
   evas_font_cache_set(evas, 0);
#endif

   ob = evas_object_rectangle_add(evas);
   evas_object_move(ob, 0, 0);
   evas_object_resize(ob, win_w, win_h);
   evas_object_color_set(ob, 0, 0, 0, 255);
   evas_object_show(ob);

   ob = evas_object_image_add (evas);
   evas_object_image_file_set(ob, IM "track_bg.png", NULL);
   evas_object_move(ob, 0, 0);
   evas_object_resize(ob, WIDTH/2, 3*HEIGHT/16);
   evas_object_image_fill_set(ob, 0, 0, WIDTH/2, HEIGHT/8);
   evas_object_layer_set(ob, 0);
   evas_object_color_set (ob, 240, 240, 240, 255);
   evas_object_show(ob);

   ob = evas_object_text_add (evas);
   evas_object_text_font_set (ob, FONT, FONT_SIZE * 3.6);
   evas_object_text_text_set (ob, "MetaDrums");
   evas_object_move (ob, 8, 12);
   evas_object_layer_set (ob, 1);
   evas_object_color_set (ob, 60, 60, 60, 200);
   evas_object_pass_events_set (ob, 1);
   evas_object_show (ob);

   ob = evas_object_text_add (evas);
   evas_object_text_font_set (ob, FONT, FONT_SIZE);
   evas_object_text_text_set (ob, "Banana code by Conrad Parker 2003");
   evas_object_move (ob, 12, 5 * HEIGHT/32);
   evas_object_layer_set (ob, 1);
   evas_object_color_set (ob, 60, 60, 60, 200);
   evas_object_pass_events_set (ob, 1);
   evas_object_show (ob);

   setup_tempo_slider ();

   setup_vcf303 ();

   setup_scrub ();

   setup_delay (0, delay1_fx, &delay1_x);
   setup_delay (1, delay2_fx, &delay2_x);

#if 0
   setup_track (0, "Clave", clave_sf, 1.0, IM "drum.png");
   setup_track (1, "Clap", clap_sf, 1.0, IM "drum.png");
#else
   setup_track (0, "Square B", square2, 0.1, IM "note.png");
   setup_track (1, "Square A", square1, 0.1, IM "note.png");
#endif
   setup_track (2, "Open Hat", openhat_sf, 1.0, IM "drum.png");
   setup_track (3, "Closed Hat", closedhat_sf, 1.0, IM "drum.png");
   setup_track (4, "Conga High", congahigh_sf, 1.0, IM "drum.png");
   setup_track (5, "Conga Low", congalow_sf, 1.0, IM "drum.png");
   setup_track (6, "Snare", snare_sf, 1.0, IM "drum.png");
   setup_track (7, "Kick", kick_sf, 1.0, IM "drum.png");

#ifdef TINY_CURSOR
   cursor = evas_object_rectangle_add (evas);
   evas_object_resize (cursor, 2, HEIGHT/8);
#else
   cursor = evas_object_image_add (evas);
   evas_object_image_file_set (cursor, IM "cursor.png", NULL);
   evas_object_image_fill_set (cursor, 0, 0, 32, 3*win_h/4);
   evas_object_resize (cursor, 32, 3*win_h/4);
#endif
   evas_object_move (cursor, 0, win_h/4);
   evas_object_color_set (cursor, 200, 240, 200, 255);
   evas_object_pass_events_set (cursor, 1);
   evas_object_show (cursor);
}

void
setup (void)
{
  setup_decks ();
  setup_gui ();
}

void
play (void)
{
  MashCount length, offset, n = 0;
  static int i = 0;

  length = mash_length (mash, mix_deck);
  offset = mash_tell (mash, mix_deck);

  if (offset > length) {
    n = 0;
  } else {
    n = MIN (1024, length - offset);
    n = mash_process (mash, mix_deck, n, MashNone, MashNone);
    if (n == -1) {
      printf ("mash_process error\n");
    }
  }

  if (n == 0) {
    mash_seek (mash, mix_deck, 0, SEEK_SET);
    offset = 0;
  }

#ifdef TINY_CURSOR
  evas_object_move (cursor, (win_w * offset/length) - 2, win_h/4);
#else
  evas_object_move (cursor, (win_w * offset/length) - 32, win_h/4);
#endif
}
